## This package handles PBS stuff...
## It uses PBSNode.pm and PBSJob.pm to store PBS nodes and Jobs descriptions
package PBS;

use strict;
use warnings;
use monika::PBSNode;
use monika::PBSJob;

## Class constructor
sub new {
  my $proto = shift;
  my $class = ref($proto) || $proto;
  my $self = {};
  $self->{ALLNODES} = {};
  $self->{PROPERTIES} = {};
  $self->{ALLJOBS} = {};
  bless ($self,$class);
  return $self;
}

## return all nodes got from the pbsnodes cmd
sub allnodes {
  my $self = shift;
  return $self->{ALLNODES};
}

sub properties {
  my $self = shift;
  return $self->{PROPERTIES};
}

## call pbsnodes cmd to retrieve PBS nodes description and store them in the ALLNODES hash.
## also store nodes by pbs property (resource) in the PROPERTIES hash
sub pbsnodes {
  my $self = shift;
  my $batchscheduler = shift;
  my $pbsnodesCmd = Conf::myself->pbsnodesCmd." |";
  unless (defined $pbsnodesCmd) {
    $pbsnodesCmd = "pbsnodes -a |";
  }
  open PBSNODES,$pbsnodesCmd;
  my $pbsnodesStr = join "",<PBSNODES>;
  close PBSNODES;
  chomp $pbsnodesStr;
  my @nodesStrs = (split /^\s*\n/m,$pbsnodesStr);
  foreach my $nodeStr (@nodesStrs) {
    $nodeStr =~ s/(\n     )(.)/$1$2$2/gm;
    my @lines = split /\n     [^\s]/,$nodeStr;
    my $name = shift @lines or die "missing a node name !";
    my $node = new PBSNode($name);
    foreach my $line (@lines) {
      $line =~ s/\n\s*//gm;
      my ($key,$value) = $line =~ /([^=]+) = (.*)/ or die "Malformed line: $line";
      $node->set($key,$value);
    }
    $self->allnodes()->{$node->displayname} = $node;
    foreach my $property ($node->properties()) {
			push @{$self->properties()->{$property}},$node->displayname;
    }
  }
}

## call qstat to retrieve PBS jobs description and store them in the ALLJOBS hash.
sub qstat {
  my $self = shift;
  my $cgi = shift;
  my $qstatCmd = Conf::myself->qstatCmd." |";
  unless (defined $qstatCmd) {
    $qstatCmd = "qstat -f |";
  }
  open QSTAT,$qstatCmd;
  my $qstatStr = join "",<QSTAT>;
  close QSTAT;
  chomp $qstatStr;
  my @nodesStrs = (split /^\s*\nJob Id: /m,$qstatStr);
  foreach my $nodeStr (@nodesStrs) {
    $nodeStr =~ s/(\n    )(.)/$1$2$2/gm;
    my @lines = split /\n    [^\s]/,$nodeStr;
    my $jobId = shift @lines or die "missing a job Id";
    $jobId =~ s/Job Id: //;
    my $job = PBSJob->new($jobId);
    foreach my $line (@lines) {
      $line =~ s/\n\s*//gm;
      my ($key,$value) = $line =~ /([^=]+) = (.*)/ or die "Malformed line: $line";
      $job->set($key,$value,$cgi);
    }
    $self->alljobs()->{$job->numId} = $job;
  }
  return 1;
}

## return nodes verifying a property
sub nodelistByProperty {
  my $self = shift;
  my $property = shift;
  return $self->properties()->{$property};
}

## return all jobs
sub alljobs {
  my $self = shift;
  return $self->{ALLJOBS};
}

## compute a summary of the usage of PBS nodes
sub nodecount {
  my $self = shift;
  my $total = 0;
  my $totalcpu = 0;
  my $busy = 0;
  my $busycpu = 0;
  my $free = 0;
  my $freecpu = 0;
  my $other = 0;
  my @nodes = values %{$self->allnodes};
  $total = $#nodes +1;
  foreach my $node (@nodes) {
    $totalcpu += $node->cpus;
    if ($node->state() eq "free") {
      $free++;
      my %jobs = $node->jobs();
      my @busycpus = keys %jobs;
      $freecpu += $node->cpus - ($#busycpus+1);
      $busycpu += $#busycpus+1;
    } elsif ($node->state() =~ /job/) {
      $busy++;
      #$busycpu+= $node->cpus;
      my %jobs = $node->jobs();
      my @busycpus = keys %jobs;
      $freecpu += $node->cpus - ($#busycpus+1);
      $busycpu += $#busycpus+1;
    } else {
      $other++;
    }
  }
  return ($total,$totalcpu,$free,$freecpu,$busy,$busycpu,$other);
}

#}

## print a HTML tables describing current PBS jobs.
sub htmlJobTable {
  my $self = shift;
  my $cgi = shift;
  my $batch = shift;
  my $output = "";
#  $output .= $cgi->start_form();
  $output .= $cgi->start_table({-border=>"1", -align => "center"});
  my $alljobs = $self->alljobs();
  my @keys = keys %$alljobs;
  if ($#keys < 0) {
    $output .= $cgi->Tr($cgi->td("No job currently in queues"));
  } else {
    if ($batch eq "OAR"){
        $output .= $cgi->start_Tr();
        $output .= $cgi->td({-align => "center"},"Id");
        $output .= $cgi->td({-align => "center"},"User");
        $output .= $cgi->td({-align => "center"},"State");
        $output .= $cgi->td({-align => "center"},"Queue");
        $output .= $cgi->td({-align => "center"},"NbNodes");
        $output .= $cgi->td({-align => "center"},"Weight");
        $output .= $cgi->td({-align => "center"},"Type");
        $output .= $cgi->td({-align => "center"},"Properties");
        $output .= $cgi->td({-align => "center"},"Reservation");
        $output .= $cgi->td({-align => "center"},"Walltime");
        $output .= $cgi->td({-align => "center"},"Submission Time");
        $output .= $cgi->td({-align => "center"},"Start Time");
#        $output .= $cgi->td({-align => "center"},"Comment");
        $output .= $cgi->td({-align => "center"},"Scheduled Start");
        $output .= $cgi->end_Tr();
    }
    @keys = sort {$a <=> $b} @keys;
    foreach my $key (@keys) {
      $output .= $alljobs->{$key}->htmlTableRow($cgi,$batch);
    }
  }
  $output .= $cgi->end_table();
#  $output .= $cgi->endform();

  return $output;
}

## print a HTML summary table of the current usage of the PBS nodes.
sub htmlSummaryTable {
  my $self = shift;
  my $cgi = shift;
  my $batch = shift;
  my $output = "";
  my ($total,$totalcpu,$free,$freecpu,$busy,$busycpu,$other) = $self->nodecount();
  $output .= $cgi->start_table({-border=>"1",
			   -align =>"center"
			  });
  $output .= $cgi->start_Tr({-valign=>"middle",
			-align=>"center"
		       });
  $output .= $cgi->td($cgi->i("$batch node status"));
  $output .= $cgi->td($cgi->b("Free"));
  $output .= $cgi->td($cgi->b("Busy"));
  $output .= $cgi->td($cgi->b("Total"));
  $output .= $cgi->end_Tr();

  $output .= $cgi->start_Tr({-valign=>"middle",
			-align=>"center"
		       });
  $output .= $cgi->td($cgi->b("Nodes"));
  $output .= $cgi->td([$free,$busy,$total]);
  $output .= $cgi->end_Tr();

  if ($total != $totalcpu) {
    $output .= $cgi->start_Tr({-valign=>"middle",
			  -align=>"center"
			 });
    $output .= $cgi->td($cgi->b("CPUs"));
    $output .= $cgi->td([$freecpu,$busycpu,$totalcpu]);
    $output .= $cgi->end_Tr();
  }
  $output .= $cgi->end_table();
  return $output;
}

sub htmlPropertyChooser {
  my $self = shift;
  my $cgi = shift;
  my $batchName = shift;
  my $output = "";
	# do not show hidden properties...
  my @checkboxes = ();
	my @hiddenProperties = Conf::myself()->hiddenProperties();
	foreach my $p (sort keys %{$self->properties()}) {
		my $hidden = undef;
		foreach my $h (@hiddenProperties) {
			$p =~ /^$h=/ and $hidden = 1;
		}
		defined $hidden or push @checkboxes, $p;
	}		
  if ($#checkboxes >= 0) {
    $output .= $cgi->start_div({ -align => "center" });
    $output .= $cgi->start_form({ -method => "get" });
    $output .= $cgi->b("$batchName properties:");
    $output .= $cgi->checkbox_group({
				    -name=> 'props',
				    -values=> \@checkboxes,
				    -columns => 5,
				     -title => "click to select property"
				    });
    $output .= $cgi->submit("Action","Display nodes for these properties");
    $output .= $cgi->end_div();
    $output .= $cgi->endform();
  }
  return $output;
}

sub htmlNodeByProperty {
  my $self = shift;
  my $cgi = shift;
  my $output = "";
  my @selected = sort $cgi->param('props');
  foreach my $prop (@selected) {
    $output .= $cgi->h3({-align => "center"},
			"Reservations for property $prop:");
    $output .= $cgi->nodeReservationTable($self->allnodes(),$self->nodelistByProperty($prop));
  }
  return $output;
}

## that's all.
return 1;

